//  Backpanel P20 with 5 braille cells
//  -----------------------------------------------------------------------------------------------             7 +200V  <-------------------------- DC-DC +185V
// |  cell 0   |   cell 1  |  cell 2   |  cell 3   |  cell 4   |  cell 5   |  cell 6   |  cell 7   | <-- Cable  6 n.c.
//  ------------------------------------------------------------------------------------------------            5 GND    <----   Arduino GND    ---> DC-DC GND
// |  1 oo 8   |  1 oo 8   |  1 oo 8   |  1 oo 8   |  1 oo 8   |  values of the pins               |            4 CLK    <----   Arduino pin 5
// |  2 oo 16  |  2 oo 16  |  2 oo 16  |  2 oo 16  |  2 oo 16  |                                   |            3 STRB   <----   Arduino pin 4
// |  4 oo 32  |  4 oo 32  |  4 oo 32  |  4 oo 32  |  4 oo 32  |                                   |            2 Din    <----   Arduino pin 3
// | 64 oo 128 | 64 oo 128 | 64 oo 128 | 64 oo 128 | 64 oo 128 |                                   |            1 +5V    <----   Arduino +5V    ---> DC-DC +5V
//  -----------------------------------------------------------------------------------------------                              Arduino pin 2  ---> DC-DC /ON
//
// YouTube video: https://www.youtube.com/watch?v=3zqth08wQBs
// P20 cell:      http://web.metec-ag.de/P20.pdf
// DC-DC:         http://web.metec-ag.de/DCDC%20Converter%205to200V.pdf
// Contact:       ab@metec-ag.de

#define CLK 6
#define DT 8
#define SW 7

#include "GyverEncoder.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"

Encoder enc1(CLK, DT, SW);
SoftwareSerial mySoftwareSerial(10, 9); // RX, TX
DFRobotDFPlayerMini myDFPlayer;

const int ON     =  A0;  // DC-DC Converter ON Pin. You can use it to switch off the cell power in a sleeping mode
const int DATA   =  2;  // Backpanel Pin 2 (Din)
const int STROBE =  3;  // Backpanel Pin 3 (STRB)
const int CLOCK  =  5;  // Backpanel Pin 4 (CLK)
//const int POTI   =  0;  // Optional. There is an potentiometer on analog pin 0

int stepVar = 192;
int cellNum = 7;
boolean stateOn = 0;

const int cellCount = 8;
byte cells[cellCount];


void setup()
{

  mySoftwareSerial.begin(9600);
  Serial.begin(115200);
  enc1.setType(TYPE2);

  pinMode(ON,     OUTPUT);
  pinMode(DATA,   OUTPUT);
  pinMode(STROBE, OUTPUT);
  pinMode(CLOCK,  OUTPUT);

  if (!myDFPlayer.begin(mySoftwareSerial)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
    while (true);
  }
  myDFPlayer.setTimeOut(500);
  myDFPlayer.volume(30);

  //  digitalWrite(ON, 0);  // 0=ON, 1=OFF
}


void loop()
{
  // left cell run all pins, next cell blinks and the others are running horizonal
  // Set the pins, send it and wait...
  //  cells[0] = 1;   cells[1] = 0;      cells[2] = 1 + 2 + 4 + 64;    cells[3] = 0;                 cells[4] = 0;                 Flush(); Wait();
  //  cells[0] = 2;   cells[1] = 255;    cells[2] = 8 + 16 + 32 + 128; cells[3] = 0;                 cells[4] = 0;                 Flush(); Wait();
  //  cells[0] = 4;   cells[1] = 0;      cells[2] = 0;                 cells[3] = 1 + 2 + 4 + 64;    cells[4] = 0;                 Flush(); Wait();
  //  cells[0] = 8;   cells[1] = 255;    cells[2] = 0;                 cells[3] = 8 + 16 + 32 + 128; cells[4] = 0;                 Flush(); Wait();
  //  cells[0] = 16;  cells[1] = 0;      cells[2] = 0;                 cells[3] = 0;                 cells[4] = 1 + 2 + 4 + 64;    Flush(); Wait();
  //  cells[0] = 32;  cells[1] = 255;    cells[2] = 0;                 cells[3] = 0;                 cells[4] = 8 + 16 + 32 + 128; Flush(); Wait();
  //  cells[0] = 64;  cells[1] = 0;      cells[2] = 0;                 cells[3] = 0;                 cells[4] = 0;                 Flush(); Wait();
  //  cells[0] = 128; cells[1] = 255;    cells[2] = 0;                 cells[3] = 0;                 cells[4] = 0;                 Flush(); Wait();

  //cells[7] = B11111111; Flush();  delay(1000); // 0
  //cells[7] = B11111110; Flush();  delay(1000); // dot 8
  //cells[7] = B11111101; Flush();  delay(1000); // dot 7
  //cells[7] = B11111011; Flush();  delay(1000); // dot 6
  //cells[7] = B11110111; Flush();  delay(1000); // dot 5
  //cells[7] = B11101111; Flush();  delay(1000); // dot 4
  //cells[7] = B11011111; Flush();  delay(1000); // dot 3
  //cells[7] = B10111111; Flush();  delay(1000); // dot 2
  //cells[7] = B01111111; Flush();  delay(1000); // dot 1

    enc1.tick();
    if (enc1.isTurn()) {
      cells[7] = getLetter(stepVar);
//      if (stepVar > 1) cells[cellNum]++;
      Flush();
      Serial.print(stepVar);
      Serial.print("  ");
      Serial.println(cells[cellNum]);
    }
    if (enc1.isRight()) {
      stepVar++;
      if (stepVar > 200) stepVar = 192;
    }
    if (enc1.isLeft()) {
      stepVar--;
      if (stepVar < 192) stepVar = 200;
    }
    if (enc1.isPress()) {
      myDFPlayer.playFolder(01, (stepVar - 192));
      Serial.println(stepVar);
      //    stateOn = !stateOn;
      //    digitalWrite(ON, stateOn);  // 0=ON, 1=OFF
      //    if (stateOn) Serial.print("Off");
      //    else Serial.print("On");
    }
}

byte getLetter(int ascii) {
  switch (ascii) {
    case 168:
      return B01111011; // Ё
      break;
    case 192:
      return B01111111; // А
      break;
    case 193:
      return B00111111; // Б
      break;
    case 194:
      return B10100011; // В
      break;
    case 195:
      return B00100111; // Г
      break;
    case 196:
      return B01100111; // Д
      break;
    case 197:
      return B01110111; // Е
      break;
    case 198:
      return B10100111; // Ж
      break;
    case 199:
      return B01010011; // З
      break;
    case 200:
      return B10101111; // И
      break;
    case 201:
      return B01111111; // Й начать отсюда
      break;
    case 202:
      return B01111111; // К
      break;
    case 203:
      return B01111111; // Л
      break;
    case 204:
      return B01111111; // М
      break;
    case 205:
      return B00111111; // Н
      break;
    case 206:
      return B10100011; // О
      break;
    case 207:
      return B01111111; // П
      break;
    case 208:
      return B01111111; // Р
      break;
    case 209:
      return B01111111; // С
      break;
    case 210:
      return B01111111; // Т
      break;
    case 211:
      return B01111111; // У
      break;
    case 212:
      return B01111111; // Ф
      break;
    case 213:
      return B01111111; // Х
      break;
    case 214:
      return B01111111; // Ц
      break;
    case 215:
      return B01111111; // Ч
      break;
    case 216:
      return B01111111; // Ш
      break;
    case 217:
      return B01111111; // Щ
      break;
    case 218:
      return B01111111; // Ъ
      break;
    case 219:
      return B01111111; // Ы
      break;
    case 220:
      return B01111111; // Ь
      break;
    case 221:
      return B01111111; // Э
      break;
    case 222:
      return B01111111; // Ю
      break;
    case 223:
      return B01111111; // Я
      break;
      //default:
      // код для выполнения
  }

}

void Wait()
{
  // optional. read the value from the poti on analog pin 0
  delay(300); //analogRead(POTI) / 2 );
}




// Send the data
void Flush ()
{
  // This example is for one P20 backpanel. If you are using others you have to change the bit order!
  // P20: 6,7,2,1,0,5,4,3
  // P16: 6,2,1,0,7,5,4,3
  // B11: 0,1,2,3,4,5,6,7
  // Rotate the bytes from the right side. Cell 0 comes first and is on the left position.

  for (int i = 0; i < cellCount; i++)
  {
    if ( bitRead(cells[i], 0) )   { //6
      digitalWrite(DATA, 0);
    }
    else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------

    if ( bitRead(cells[i], 1) )   { //7
      digitalWrite(DATA, 0);
    }
    else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------

    if ( bitRead(cells[i], 2) )   { //2
      digitalWrite(DATA, 0);
    }
    else {
      digitalWrite(DATA, 1);
    } digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------

    if ( bitRead(cells[i], 3) )   { //1
      digitalWrite(DATA, 0);
    } else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------
    if ( bitRead(cells[i], 4) )   { //0
      digitalWrite(DATA, 0);
    } else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------

    if ( bitRead(cells[i], 5) )   { //5
      digitalWrite(DATA, 0);
    } else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------

    if ( bitRead(cells[i], 6) )   { //4
      digitalWrite(DATA, 0);
    } else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------

    if ( bitRead(cells[i], 7) )   { //3
      digitalWrite(DATA, 0);
    } else {
      digitalWrite(DATA, 1);
    }
    digitalWrite(CLOCK, 1); digitalWrite(CLOCK, 0);
    //--------------------------------------------
  }

  digitalWrite(STROBE, 1);  // Strobe on
  digitalWrite(STROBE, 0);  // Strobe off
}
